/*
 *	Copied from libHX/string.c:
 */
/*
 *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 1999 - 2010
 *
 *	This file is part of libHX. libHX is free software; you can
 *	redistribute it and/or modify it under the terms of the GNU
 *	Lesser General Public License as published by the Free Software
 *	Foundation; either version 2.1 or 3 of the License.
 */
/*
 *	Copied and changed by jengelh for muuid, as LDAP escaping was not
 *	in a released libHX tarball at the time of writing muuid.
 */
#include <stdbool.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <libHX/defs.h>
#include <libHX/string.h>

static size_t muuid_qsize_backslash(const char *s, const char *qchars,
    unsigned int cost)
{
	const char *p = s;
	size_t n = strlen(s);

	while ((p = strpbrk(p, qchars)) != NULL) {
		n += cost;
		++p;
	}
	return n;
}

static char *muuid_quote_ldap(char *dest, const char *src, const char *qc)
{
	static const char hex[] = "0123456789ABCDEF";
	char *ret = dest;
	size_t len;

	while (*src != '\0') {
		len = strcspn(src, qc);
		if (len > 0) {
			memcpy(dest, src, len);
			dest += len;
			src  += len;
			if (*src == '\0')
				break;
		}
		*dest++ = '\\';
		*dest++ = hex[(*src >> 4) & 0x0F];
		*dest++ = hex[*src++ & 0x0F];
	}

	*dest = '\0';
	return ret;
}

extern char *muuid_strquote(const char *, char **);	/* temporary */

char *muuid_strquote(const char *src, char **free_me)
{
	static const char flt_chars[] = "\n*()\\";
	bool do_quote;
	char *tmp;

	do_quote = strpbrk(src, flt_chars) != NULL;
	/*
	 * free_me == NULL implies that we always allocate, even if
	 * there is nothing to quote.
	 */
	if (free_me != NULL) {
		free(*free_me);
		*free_me = NULL;
		if (!do_quote)
			return const_cast1(char *, src);
	} else {
		if (!do_quote)
			return HX_strdup(src);
		free_me = &tmp;
	}

	*free_me = malloc(muuid_qsize_backslash(src, flt_chars, 2) + 1);
	if (*free_me == NULL)
		return NULL;

	return muuid_quote_ldap(*free_me, src, flt_chars);
}
